coreutils
[Top][All Lists]
Advanced

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

[PATCH] tests: run df tests with /proc/self/mountinfo


From: Pádraig Brady
Subject: [PATCH] tests: run df tests with /proc/self/mountinfo
Date: Thu, 30 Apr 2015 11:38:27 +0100

* tests/df/no-mtab-status.sh: getmntent is no longer called
when /proc/self/mountinfo is present, thus causing the test
to be skipped.  Therefore wrap fopen() to ignore mountinfo,
and use the test genmntent table instead.
* tests/df/skip-duplicates.sh: Likewise.
---
 tests/df/no-mtab-status.sh  | 34 +++++++++++++++++++++++++++-------
 tests/df/skip-duplicates.sh | 36 ++++++++++++++++++++++++++++--------
 2 files changed, 55 insertions(+), 15 deletions(-)

diff --git a/tests/df/no-mtab-status.sh b/tests/df/no-mtab-status.sh
index 49d1255..be1cecc 100755
--- a/tests/df/no-mtab-status.sh
+++ b/tests/df/no-mtab-status.sh
@@ -32,20 +32,40 @@ grep '^#define HAVE_GETMNTENT 1' $CONFIG_HEADER > /dev/null 
\
 
 # Simulate "mtab" failure.
 cat > k.c <<EOF || framework_failure_
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <errno.h>
 #include <mntent.h>
-#include "$CONFIG_HEADER"
+#include <string.h>
+#include <dlfcn.h>
 
-#ifdef MOUNTED_PROC_MOUNTINFO
-# include <libmount/libmount.h>
-struct libmnt_table *mnt_new_table_from_file(const char *filename)
+FILE* fopen(const char *path, const char *mode)
 {
-  /* Returning NULL here will get read_file_system_list()
+  static FILE* (*fopen_func)(char const *, char const *);
+
+  /* get reference to original (libc provided) fopen */
+  if (!fopen_func)
+    {
+      fopen_func = (FILE*(*)(char const *, char const *))
+                   dlsym(RTLD_NEXT, "fopen");
+      if (!fopen_func)
+        {
+          fprintf (stderr, "Failed to find fopen()\n");
+          errno = ESRCH;
+          return NULL;
+        }
+    }
+
+  /* Returning ENOENT here will get read_file_system_list()
      to fall back to using getmntent() below.  */
-  return NULL;
+  if (strcmp (path, "/proc/self/mountinfo") == 0)
+    {
+      errno = ENOENT;
+      return NULL;
+    }
+  else
+    return fopen_func(path, mode);
 }
-#endif
 
 struct mntent *getmntent (FILE *fp)
 {
diff --git a/tests/df/skip-duplicates.sh b/tests/df/skip-duplicates.sh
index 2560708..e6a68fe 100755
--- a/tests/df/skip-duplicates.sh
+++ b/tests/df/skip-duplicates.sh
@@ -41,21 +41,41 @@ grep '^#define HAVE_GETMNTENT 1' $CONFIG_HEADER > /dev/null 
\
 
 # Simulate an mtab file to test various cases.
 cat > k.c <<EOF || framework_failure_
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
+#include <errno.h>
 #include <mntent.h>
-#include "$CONFIG_HEADER"
+#include <string.h>
+#include <dlfcn.h>
 
-#ifdef MOUNTED_PROC_MOUNTINFO
-# include <libmount/libmount.h>
-struct libmnt_table *mnt_new_table_from_file(const char *filename)
+FILE* fopen(const char *path, const char *mode)
 {
-  /* Returning NULL here will get read_file_system_list()
+  static FILE* (*fopen_func)(char const *, char const *);
+
+  /* get reference to original (libc provided) fopen */
+  if (!fopen_func)
+    {
+      fopen_func = (FILE*(*)(char const *, char const *))
+                   dlsym(RTLD_NEXT, "fopen");
+      if (!fopen_func)
+        {
+          fprintf (stderr, "Failed to find fopen()\n");
+          errno = ESRCH;
+          return NULL;
+        }
+    }
+
+  /* Returning ENOENT here will get read_file_system_list()
      to fall back to using getmntent() below.  */
-  return NULL;
+  if (strcmp (path, "/proc/self/mountinfo") == 0)
+    {
+      errno = ENOENT;
+      return NULL;
+    }
+  else
+    return fopen_func(path, mode);
 }
-#endif
 
 #define STREQ(a, b) (strcmp (a, b) == 0)
 
-- 
2.3.4




reply via email to

[Prev in Thread] Current Thread [Next in Thread]